using System;
using System.Data;
using System.Data.OleDb;
using DebtManagerTest.InterfaceComponents;
using System.Diagnostics;

namespace DebtManagerTest
{
	/// <summary>
	/// Summary description for CDiary.
	/// </summary>
	public class CDiary
	{
		private static OleDbDataAdapter mPendingDiaryAdapter;
		private static OleDbDataAdapter mDiaryAdapter;
		private static OleDbDataAdapter mMainAdapter;
		public static event EventHandler DiaryUpdated;

		public static void CleanUp()
		{
			if (mDiaryAdapter != null)
				mDiaryAdapter.Dispose();
			if (mMainAdapter != null)
                mMainAdapter.Dispose();
			if (mPendingDiaryAdapter != null)
                mPendingDiaryAdapter.Dispose();
		}

		public static dsDiary GetDiaryContents(bool pendingOnly)
		{
			dsDiary temp = new dsDiary();
			if (pendingOnly)
			{
				cMyDB.FillDS(temp, "Diary", PendingDiaryAdapter);
				return temp;
			}
			else
			{				
				cMyDB.FillDS(temp, "Diary", DiaryAdapter);
				return temp;
			}
		}

		public static void RefreshDiaries()
		{
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
		}

		public static void RefreshData(dsDiary theData,bool pendingOnly)
		{
			theData.Clear();
			if (pendingOnly)
			{
				cMyDB.FillDS(theData,"Diary",PendingDiaryAdapter);
			}
			else
			{
				cMyDB.FillDS(theData,"Diary",DiaryAdapter);
			}
		}

		public static void Diarise()
		{
			frmDiarise frm = new frmDiarise();
			frm.PrepForInsert();
			frm.ShowDialog();
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
			frm.Dispose();
		}

		public static void Diarise(Int32 CaseID)
		{
			frmDiarise frm = new frmDiarise();
			frm.PrepForInsert(CaseID);
			frm.ShowDialog();
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
			frm.Dispose();
		}

		public static void EditDiary(Int32 DiaryID)
		{
			frmDiarise frm = new frmDiarise();
			try
			{
				frm.PrepForEdit(DiaryID);
				frm.ShowDialog();
			}
			finally
			{
				frm.Dispose();
				if(DiaryUpdated != null)
					DiaryUpdated(null, new EventArgs());			
			}
		}

		public static void NewJournal(Int32 CaseID)
		{
			frmCompleteDiary frm = new frmCompleteDiary();
			frm.PrepForInsert(CaseID);
			frm.ShowDialog();
			frm.Dispose();
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
		}

		public static void NewJournal()
		{
			frmCompleteDiary frm = new frmCompleteDiary();
			frm.PrepForInsert();
			frm.ShowDialog();
			frm.Dispose();
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
		}

		public static void DeleteDiary(Int32 DiaryID)
		{
			dsDiary ds = new dsDiary();
			MainAdapter.SelectCommand.Parameters["ID"].Value = DiaryID;
			MainAdapter.Fill(ds, "Diary");
			ds.Diary.FindByID(DiaryID).Delete();
			MainAdapter.Update(ds, "Diary");
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
			ds.Dispose();
		}
		public static void CompleteDiary(Int32 DiaryID)
		{
			frmCompleteDiary frm = new frmCompleteDiary();
			try
			{
				frm.PrepForEdit(DiaryID);
				frm.ShowDialog();
			}
			finally
			{
				frm.Dispose();
				if(DiaryUpdated != null)
					DiaryUpdated(null, new EventArgs());
			}
		}
		public static void CompleteDiaryNow(Int32 DiaryID)
		{
			dsDiary ds = new dsDiary();
			MainAdapter.SelectCommand.Parameters["ID"].Value = DiaryID;
			MainAdapter.Fill(ds, "Diary");
			DataRow dr = ds.Diary.FindByID(DiaryID);
			
			dr.BeginEdit();
			dr["Completed"] = true;
			dr["DateActioned"] = DateTime.Today;
			dr["LastUpdated"] = DateTime.Now;
			dr["LastUser"] = CSession.User.Username;				
			dr.EndEdit();
			
			MainAdapter.Update(ds, "Diary");
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
			ds.Dispose();
		}
		public static void ReOpenDiary(Int32 DiaryID)
		{
			dsDiary ds = new dsDiary();
			MainAdapter.SelectCommand.Parameters["ID"].Value = DiaryID;
			MainAdapter.Fill(ds, "Diary");
			DataRow dr = ds.Diary.FindByID(DiaryID);
			
			dr.BeginEdit();
			dr["Completed"] = false;
			dr["DateActioned"] = DBNull.Value;
			dr["LastUpdated"] = DateTime.Now;
			dr["LastUser"] = CSession.User.Username;				
			dr.EndEdit();
			
			MainAdapter.Update(ds, "Diary");
			if(DiaryUpdated != null)
				DiaryUpdated(null, new EventArgs());
			ds.Dispose();
		}

		public static OleDbDataAdapter PendingDiaryAdapter
		{
			get
			{
				if (mPendingDiaryAdapter==null)
				{
					mPendingDiaryAdapter = new OleDbDataAdapter();
					mPendingDiaryAdapter.SelectCommand = new OleDbCommand("Select Diary.*, ActionTypeLookup.Description as ActionTypeCaption, Cases.AccountNumber as CaseCaption, ActionTypeLookup.PictureType as PicType FROM Cases Right JOIN (Diary Left JOIN ActionTypeLookup ON ActionTypeLookup.ID = Diary.ActionType) ON Cases.ID = Diary.CaseID Where (UserID = ? or UserID is null) And ActionDate <= Now And Completed = false Order By ActionDate desc");
					mPendingDiaryAdapter.SelectCommand.Parameters.Add(CUtil.NewParam("user", OleDbType.Integer, CSession.User.UserID));
				}
				else
				{
					mPendingDiaryAdapter.SelectCommand.Parameters["user"].Value = CSession.User.UserID;
				}
				return mPendingDiaryAdapter;
			}
		}
		public static OleDbDataAdapter MainAdapter
		{
			get
			{
				if (mMainAdapter==null)
				{
					mMainAdapter = new OleDbDataAdapter();
					// 
					// SelectCommand
					// 
					mMainAdapter.SelectCommand = new OleDbCommand("Select Diary.* from Diary Where ID = ?");
					mMainAdapter.SelectCommand.Connection = cMyDB.NewConnection();
					mMainAdapter.SelectCommand.Parameters.Add(new OleDbParameter("id", OleDbType.Integer));
					
					mMainAdapter.InsertCommand = new OleDbCommand();
					// 
					// InsertCommand
					// 
					mMainAdapter.InsertCommand.CommandText = "INSERT INTO Diary(ActionDate, ActionType, CaseID, Completed, Created, CreateUser," +
							" DateActioned, Description, LastUpdated, LastUser, Notes, UserID) VALUES (?, ?, " +
							"?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
					mMainAdapter.InsertCommand.Connection = mMainAdapter.SelectCommand.Connection;
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("ActionDate", OleDbType.DBDate, 0, "ActionDate"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("ActionType", OleDbType.Integer, 0, ParameterDirection.Input, false, ((System.Byte)(10)), ((System.Byte)(0)), "ActionType", DataRowVersion.Current, null));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("CaseID", OleDbType.Integer, 0, ParameterDirection.Input, true, ((System.Byte)(10)), ((System.Byte)(0)), "CaseID", DataRowVersion.Current, null));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("Completed", OleDbType.Boolean, 2, "Completed"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("Created", OleDbType.DBDate, 0, "Created"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("CreateUser", OleDbType.VarWChar, 12, "CreateUser"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("DateActioned", OleDbType.DBDate, 0, "DateActioned"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("Description", OleDbType.VarWChar, 255, "Description"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("LastUpdated", OleDbType.Date, 0, "LastUpdated"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("LastUser", OleDbType.VarWChar, 12, "LastUser"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("Notes", OleDbType.VarWChar, 0, "Notes"));
					mMainAdapter.InsertCommand.Parameters.Add(new OleDbParameter("UserID", OleDbType.Integer, 0, ParameterDirection.Input, true, ((System.Byte)(10)), ((System.Byte)(0)), "UserID", DataRowVersion.Current, null));
					// 
					// UpdateCommand
					// 
					mMainAdapter.UpdateCommand = new OleDbCommand();
                    mMainAdapter.UpdateCommand.Connection = mMainAdapter.SelectCommand.Connection;
					mMainAdapter.UpdateCommand.CommandText = @"UPDATE Diary SET ActionDate = ?, ActionType = ?, CaseID = ?, Completed = ?, Created = ?, CreateUser = ?, DateActioned = ?, Description = ?, LastUpdated = ?, LastUser = ?, Notes = ?, UserID = ? WHERE (ID = ?) AND (LastUpdated = ? OR ? IS NULL AND LastUpdated IS NULL)";
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("ActionDate", OleDbType.DBDate, 0, "ActionDate"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("ActionType", OleDbType.Integer, 0, ParameterDirection.Input, false, ((System.Byte)(10)), ((System.Byte)(0)), "ActionType", DataRowVersion.Current, null));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("CaseID", OleDbType.Integer, 0, ParameterDirection.Input, true, ((System.Byte)(10)), ((System.Byte)(0)), "CaseID", DataRowVersion.Current, null));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Completed", OleDbType.Boolean, 2, "Completed"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Created", OleDbType.DBDate, 0, "Created"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("CreateUser", OleDbType.VarWChar, 12, "CreateUser"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("DateActioned", OleDbType.DBDate, 0, "DateActioned"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Description", OleDbType.VarWChar, 255, "Description"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("LastUpdated", OleDbType.Date, 0, "LastUpdated"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("LastUser", OleDbType.VarWChar, 12, "LastUser"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Notes", OleDbType.VarWChar, 0, "Notes"));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("UserID", OleDbType.Integer, 0, ParameterDirection.Input, true, ((System.Byte)(10)), ((System.Byte)(0)), "UserID", DataRowVersion.Current, null));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_ID", OleDbType.Integer, 0, System.Data.ParameterDirection.Input, false, ((System.Byte)(10)), ((System.Byte)(0)), "ID", DataRowVersion.Original, null));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_LastUpdated", OleDbType.Date, 0, ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "LastUpdated", DataRowVersion.Original, null));
					mMainAdapter.UpdateCommand.Parameters.Add(new OleDbParameter("Original_LastUpdated1", OleDbType.Date, 0, ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "LastUpdated", DataRowVersion.Original, null));
					// 
					// DeleteCommand
					// 
					mMainAdapter.DeleteCommand = new OleDbCommand();
					mMainAdapter.DeleteCommand.Connection = mMainAdapter.SelectCommand.Connection;
					mMainAdapter.DeleteCommand.CommandText = @"DELETE FROM Diary WHERE (ID = ?) And (LastUpdated = ? OR ? IS NULL AND LastUpdated IS NULL)";
					mMainAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_ID", OleDbType.Integer, 0, ParameterDirection.Input, false, ((System.Byte)(10)), ((System.Byte)(0)), "ID", DataRowVersion.Original, null));
					mMainAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_LastUpdated", OleDbType.Date, 0, ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "LastUpdated", DataRowVersion.Original, null));
					mMainAdapter.DeleteCommand.Parameters.Add(new OleDbParameter("Original_LastUpdated1", OleDbType.Date, 0, ParameterDirection.Input, false, ((System.Byte)(0)), ((System.Byte)(0)), "LastUpdated", DataRowVersion.Original, null));

					mMainAdapter.RowUpdated += new OleDbRowUpdatedEventHandler(Main_RowUpdated);
				}
				else
				{
				}
				return mMainAdapter;
			}
		}

		public static void Main_RowUpdated(object sender, OleDbRowUpdatedEventArgs e)
		{
			if (e.RecordsAffected == 0)
			{
				if (e.Errors is DBConcurrencyException)
				{
					dsDiary current = new dsDiary();
					CDiary.MainAdapter.SelectCommand.Parameters["id"].Value = e.Row["id"];
					CDiary.MainAdapter.Fill(current, "Diary");

					if (current.Diary.Rows.Count > 0)	
					{
						RecordConflict rcFrm = new RecordConflict();
						rcFrm.Compare(current.Diary.Rows[0], e.Row);

						rcFrm.ShowDialog();

						if (rcFrm.DialogResult != System.Windows.Forms.DialogResult.OK)
							e.Status = UpdateStatus.SkipCurrentRow;

						rcFrm.Dispose();

						if (e.Status != UpdateStatus.SkipCurrentRow)
						{
							current.Diary.Rows[0].BeginEdit();
							Debug.WriteLine(current.Diary.Rows[0]["LastUpdated",DataRowVersion.Original]);
							foreach (DataColumn dc in current.Diary.Columns)
							{
								current.Diary.Rows[0][dc.ColumnName] = e.Row[dc.ColumnName];
							}
							current.Diary.Rows[0].EndEdit();
							Debug.WriteLine(current.Diary.Rows[0]["LastUpdated",DataRowVersion.Original]);
							MainAdapter.Update(current, "Diary");
							e.Status = UpdateStatus.Continue;
						}
					}
					current.Dispose();
				}
			}
		}

		public static OleDbDataAdapter DiaryAdapter
		{
			get
			{
				if (mDiaryAdapter==null)
				{
					mDiaryAdapter = new OleDbDataAdapter();
					mDiaryAdapter.SelectCommand = new OleDbCommand("Select Diary.*, ActionTypeLookup.Description as ActionTypeCaption, '' as CaseCaption, ActionTypeLookup.PictureType as PicType from Diary Left Join ActionTypeLookup On Diary.ActionType = ActionTypeLookup.ID Where (CaseID = ? and (UserID = ? or UserID is null or Completed = true)) order by ActionDate desc");
					mDiaryAdapter.SelectCommand.Parameters.Add(CUtil.NewParam("case", OleDbType.Integer, CSession.CaseID));
					mDiaryAdapter.SelectCommand.Parameters.Add(CUtil.NewParam("user", OleDbType.Integer, CSession.User.UserID));
				}
				else
				{
					mDiaryAdapter.SelectCommand.Parameters["case"].Value = CSession.CaseID;
					mDiaryAdapter.SelectCommand.Parameters["user"].Value = CSession.User.UserID;
				}
				return mDiaryAdapter;
			}
		}
	}
}
