using System;
using System.Data;
using System.Data.OleDb;
using CrystalDecisions.CrystalReports.Engine;

namespace DebtManagerTest
{
	/// <summary>
	/// Summary description for CReportViewer.
	/// </summary>
	public class CReportViewer
	{
		public CReportViewer(CReportDataEngine rde, ReportClass rpt)
		{
			frmReportViewer frm = new frmReportViewer();
			frm.crvMain.ReportSource = rpt;
			using (System.Data.DataSet ds = rde.GetData())
			{
				((ReportClass)frm.crvMain.ReportSource).SetDataSource(ds);
				frm.crvMain.Zoom(1);
				frm.ShowDialog();
			}
			frm.Dispose();
		}
	}

	public class V4ReportViewer
	{
		public V4ReportViewer(CReportDataEngine rde)
		{
			System.Windows.Forms.PrintPreviewDialog frm = new System.Windows.Forms.PrintPreviewDialog();
			frm.Document = new ArrearsDataReport(rde.GetData().Tables[0]);
			frm.ShowDialog();
			frm.Dispose();
		}
	}

	public class ArrearsDataReport : v4HelperControls.DataReport
	{
		public ArrearsDataReport(DataTable table)
		{
			this.DocumentName = "Arrears Report";
			ReportTitle = table.TableName + " Report";
			DataSource = table;
		
			PrintReportTotals = false;
		}
	}

	public abstract class CReportDataEngine
	{
		protected string mSQL;

		public CReportDataEngine(string SQL)
		{
			mSQL = SQL;
		}
		public CReportDataEngine()
		{}
		public abstract System.Data.DataSet GetData();
	}

	public class CasesDataEngine : CReportDataEngine
	{
		public CasesDataEngine()
		{
			mSQL = "Select * from Cases Where Archived = false";
		}
		public CasesDataEngine(string SQL)
		{
			mSQL = SQL;
		}
		public override System.Data.DataSet GetData()
		{
			dsCases dsCases1 = new dsCases();
			OleDbDataAdapter da = new OleDbDataAdapter(mSQL, cMyDB.NewConnection());
			da.Fill(dsCases1, "Cases");
			da.Dispose();
			return dsCases1;
		}
	}

	public class PinPDataEngine : CReportDataEngine
	{
		public PinPDataEngine()
		{
		}
		public override System.Data.DataSet GetData()
		{
			dsPossessionHistory dsPH = new dsPossessionHistory();
			OleDbDataAdapter da = new OleDbDataAdapter("Select ID, AccountNumber, AccountTitle, PropertyAddress, CapitalBalance, PossessionDate, IndemnityCo, IndemnityScheme, IndemnityAmount, MarketPrice, DateSold, SalePrice From Cases Where PossessionDate is not null Order By PossessionDate", cMyDB.NewConnection());
			da.Fill(dsPH, "Cases");
			
			if (dsPH.Cases.Count == 0)
				return dsPH;

			DateTime startDate = new DateTime(dsPH.Cases[0].PossessionDate.Year, dsPH.Cases[0].PossessionDate.Month, 1);
			DateTime endDate = startDate;
			int monthID = 1;
			while (endDate <= DateTime.Today)
			{
				dsPH.GraphValues.AddGraphValuesRow(endDate,0, monthID);
				dsPossessionHistory.PropertiesInMonthRow pimr = dsPH.PropertiesInMonth.NewPropertiesInMonthRow();
				pimr.MonthID = monthID;
				pimr.SetPropertyIDNull();
				dsPH.PropertiesInMonth.AddPropertiesInMonthRow(pimr);
				monthID++;
				endDate = endDate.AddMonths(1);
			}
			endDate = endDate.AddMonths(-1);

			DateTime workDate = startDate;
			DateTime monthStart = startDate;
			DateTime monthEnd = (startDate.AddMonths(1)).AddDays(-1);
			int inPossession = 0;
			bool possessed = false;
			monthID = 1;

			while (workDate <= endDate)
			{
				foreach (dsPossessionHistory.CasesRow cr in dsPH.Cases)
				{
					if (cr.PossessionDate < monthStart)
					{
						if (cr.IsDateSoldNull())
							possessed = true;
						else if (cr.DateSold >= monthStart)
							possessed = true;
					}
					else
					{
						if (cr.PossessionDate < monthEnd)
							possessed = true;
					}

					if (possessed)
					{
						inPossession++;
						dsPH.PropertiesInMonth.AddPropertiesInMonthRow(dsPH.GraphValues.FindByID(monthID),cr);
						possessed = false;
					}
				}
				dsPH.GraphValues.FindByID(monthID).NoInPossession = inPossession;
				monthID++;
				workDate = workDate.AddMonths(1);
				if (workDate <= endDate)
				{
					monthStart = workDate;
					monthEnd = (workDate.AddMonths(1)).AddDays(-1);
					inPossession = 0;
				}
			}

			da.Dispose();
			return dsPH;
		}
	}
}
