using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Data.OleDb;
using System.Diagnostics;

namespace DebtManagerTest
{
	/// <summary>
	/// Summary description for frmCapitalBalanceHistory.
	/// </summary>
	public class HistoryBaseForm : System.Windows.Forms.Form
	{
		private System.Windows.Forms.DataGrid dataGrid1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private OleDbDataAdapter mDa;
		private OleDbDataAdapter mConflictDa;
		protected DataSet mDs;
		protected string mSingleRowSQL;
		protected string mTableName;
		protected string mFieldName;
		protected string mFieldCaption;
		protected bool currencyField = true;

		protected OleDbDataAdapter DA
		{
			get
			{
				return mDa;
			}
			set
			{
				mDa = value;
				mDa.SelectCommand.Connection = cMyDB.NewConnection();
				mDa.UpdateCommand.Connection = mDa.SelectCommand.Connection;
				mDa.InsertCommand.Connection = mDa.SelectCommand.Connection;
				mDa.DeleteCommand.Connection = mDa.SelectCommand.Connection;
				mDa.RowUpdated += new OleDbRowUpdatedEventHandler(this.da_RowUpdated);
				this.dataGrid1.DataSource = this.mDs;
				this.dataGrid1.DataMember = mTableName;
			}
		}

		public HistoryBaseForm()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			//
		}
		
		public void SetupForm(Int32 CaseID)
		{
			mDa.SelectCommand.Parameters["CaseID"].Value = CaseID;
			mDs.Tables[mTableName].Columns["CaseID"].DefaultValue = CaseID;
			mDs.Tables[mTableName].Columns["AsAtDate"].DefaultValue = DateTime.Today;
			mDs.Tables[mTableName].Columns["AsAtDate"].Unique = true;
			mDa.Fill(mDs, mTableName);
			DataGridTableStyle dgts = new DataGridTableStyle();
			dgts.MappingName = mTableName;
			DataGridTextBoxColumn col1 = new DataGridTextBoxColumn();
			DataGridTextBoxColumn col2 = new DataGridTextBoxColumn();
			col1.MappingName = "AsAtDate";
			col1.HeaderText = "As At Date";
			col2.MappingName = mFieldName;
			col2.HeaderText = mFieldCaption;
			col2.Alignment = HorizontalAlignment.Right;
			col2.NullText = "";
			if (currencyField)
			{
				col2.Format = "c";
				col2.TextBox.GotFocus += new EventHandler(this.tx_focus);
				col2.TextBox.Leave += new EventHandler(this.tx_focus);
			}
			dgts.GridColumnStyles.Add(col1);
			dgts.GridColumnStyles.Add(col2);
			dataGrid1.TableStyles.Clear();
			dataGrid1.TableStyles.Add(dgts);
			dataGrid1_Resize(null, null);
		}

		private void tx_focus(object sender, EventArgs e)
		{
			if (((TextBox)sender).Text!="")
			{
				try
				{
					((TextBox)sender).Text = Decimal.Parse(((TextBox)sender).Text,
						System.Globalization.NumberStyles.Currency, null).ToString();
				}
				catch {}
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
					DA.SelectCommand.Connection.Dispose();
					DA.Dispose();
					if (mConflictDa != null)
						mConflictDa.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.dataGrid1 = new System.Windows.Forms.DataGrid();
			((System.ComponentModel.ISupportInitialize)(this.dataGrid1)).BeginInit();
			this.SuspendLayout();
			// 
			// dataGrid1
			// 
			this.dataGrid1.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.dataGrid1.DataMember = "";
			this.dataGrid1.HeaderForeColor = System.Drawing.SystemColors.ControlText;
			this.dataGrid1.Location = new System.Drawing.Point(12, 11);
			this.dataGrid1.Name = "dataGrid1";
			this.dataGrid1.Size = new System.Drawing.Size(271, 255);
			this.dataGrid1.TabIndex = 0;
			this.dataGrid1.Resize += new System.EventHandler(this.dataGrid1_Resize);
			// 
			// HistoryBaseForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(291, 273);
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.dataGrid1});
			this.FormBorderStyle = System.Windows.Forms.FormBorderStyle.SizableToolWindow;
			this.Name = "HistoryBaseForm";
			this.ShowInTaskbar = false;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterScreen;
			this.Text = "Capital Balance History";
			this.Closing += new System.ComponentModel.CancelEventHandler(this.frmCapitalBalanceHistory_Closing);
			((System.ComponentModel.ISupportInitialize)(this.dataGrid1)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		private void dataGrid1_Resize(object sender, System.EventArgs e)
		{
			if (dataGrid1.TableStyles.Count > 0)
			{
				if (dataGrid1.TableStyles[0].GridColumnStyles.Count > 1)
				{
					dataGrid1.TableStyles[0].GridColumnStyles[0].Width = (dataGrid1.ClientSize.Width) / 3;
					dataGrid1.TableStyles[0].GridColumnStyles[1].Width = (dataGrid1.ClientSize.Width) / 3;
				}
			}
		}

		private void frmCapitalBalanceHistory_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			try
			{ 
				DataRowView drv = (DataRowView)this.BindingContext[dataGrid1.DataSource, dataGrid1.DataMember].Current;
				DataRow dr = drv.Row;
				
				bool test3 = false;

				if (dr.RowState == DataRowState.Detached)
				{
					if(!dr[dr.Table.Columns[this.dataGrid1.TableStyles[0].GridColumnStyles[0].MappingName],DataRowVersion.Proposed].Equals(DateTime.Today))
						test3 = true;
					if(!dr[dr.Table.Columns[this.dataGrid1.TableStyles[0].GridColumnStyles[1].MappingName],DataRowVersion.Proposed].Equals(DBNull.Value))
						test3 = true;
				}

				bool test1 = this.dataGrid1.EndEdit(this.dataGrid1.TableStyles[0].GridColumnStyles[0], this.dataGrid1.CurrentRowIndex, false);
				bool test2 = this.dataGrid1.EndEdit(this.dataGrid1.TableStyles[0].GridColumnStyles[1], this.dataGrid1.CurrentRowIndex, false);

				if (test1 || test2 || test3)
					this.BindingContext[mDs, mTableName].EndCurrentEdit();
				else
					dr.EndEdit();
			}
			catch (Exception ex)
			{
				MessageBox.Show("This record cannot be saved at this time [" + ex.Message + "]","Invalid Data");
				e.Cancel = true;
				return;
			}
			
			foreach (DataRow dr in mDs.Tables[mTableName].Rows)
			{
				if (dr.RowState == DataRowState.Added)
				{
					dr["LastUpdated"] = DateTime.Now;
					dr["LastUser"] = CSession.User.Username;
					dr["Created"] = DateTime.Today;
					dr["CreateUser"] = CSession.User.Username;
				}
				else if(dr.RowState == DataRowState.Modified)
				{
					dr["LastUpdated"] = DateTime.Now;
					dr["LastUser"] = CSession.User.Username;
				}
				if (dr.RowState != DataRowState.Deleted)
				{
					if (dr[mFieldName].Equals(DBNull.Value) || ((((DateTime)dr["AsAtDate"]).Year == 1) && (((DateTime)dr["AsAtDate"]).Day == 1) && (((DateTime)dr["AsAtDate"]).Month == 1)))
					{ 
						MessageBox.Show("You have left some boxes blank, please fill them in, or delete the rows by clicking on the row selectors and pressing the delete key.","Invalid Data");
						e.Cancel = true;
						return;
					}
				}
			}
			mDa.Update(mDs, mTableName);
		}

		private OleDbDataAdapter ConflictDA
		{
			get 
			{
				if (mConflictDa == null)
				{
					mConflictDa = new OleDbDataAdapter();
					mConflictDa.UpdateCommand = mDa.UpdateCommand;
					mConflictDa.InsertCommand = mDa.InsertCommand;
					mConflictDa.DeleteCommand = mDa.InsertCommand;
					OleDbCommand selCmd = new OleDbCommand(mSingleRowSQL);
					selCmd.Parameters.Add("ID",OleDbType.Integer);
					selCmd.Connection = mDa.SelectCommand.Connection;
					mConflictDa.SelectCommand = selCmd;
					mConflictDa.RowUpdated += new OleDbRowUpdatedEventHandler(this.da_RowUpdated);
				}
				return mConflictDa;
			}
		}

		private void da_RowUpdated(object sender, OleDbRowUpdatedEventArgs e)
		{
			if (e.RecordsAffected == 0)
			{
				DataSet ds = new DataSet();
				
				ConflictDA.SelectCommand.Parameters["ID"].Value = e.Row["ID"];
				ConflictDA.Fill(ds);

				if (ds.Tables[0].Rows.Count > 0)
					CUtil.RowUpdatedHandler(e, ds, ConflictDA);
				else
				{
					MessageBox.Show("This record has been deleted by another user, your changes could not be saved.","Update Error");
					e.Status = UpdateStatus.SkipCurrentRow;
				}
				ds.Dispose();
			}
		}

		
	}
}
