using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using System.Diagnostics;
using System.Data.OleDb;
using Crownwood.Magic.Menus;
using UtilityLibrary.WinControls;

namespace DebtManagerTest.InterfaceComponents
{
	/// <summary>
	/// Summary description for CaseTreeViewEx.
	/// </summary>
	public class CaseTreeViewEx : TreeViewEx
	{
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.ImageList TreeImages;
		private System.Collections.Hashtable IndexNodes;
		private System.Windows.Forms.Timer scrollTimer;
		private Crownwood.Magic.Menus.MenuCommand mnuAddFav;
		private Crownwood.Magic.Menus.MenuCommand mnuDelFav;
		private CaseTreeNode DragCaseNode = null;
		private Crownwood.Magic.Menus.MenuCommand mnuDiarise;
		private Crownwood.Magic.Menus.MenuCommand mnuArchive;
		private Crownwood.Magic.Menus.MenuCommand mnuNewCase;
		private Crownwood.Magic.Menus.MenuCommand mnuDelete;
		private Crownwood.Magic.Menus.MenuCommand mnuReactivate;
		private Crownwood.Magic.Menus.MenuCommand mnuRefresh;
		private PopupMenu pm;
		public event CaseEventHandler DeleteCase;
		public event CaseEventHandler ArchiveCase;
		public event CaseEventHandler ReactivateCase;
		public event CaseEventHandler NewCase;
		private bool mSupressEvents = false;

		public CaseTreeViewEx()
		{
			// This call is required by the Windows.Forms Form Designer.
			InitializeComponent();

			// TODO: Add any initialization after the InitForm call
			this.ImageList = TreeImages;
			pm = new PopupMenu();
			pm.MenuCommands.Add(mnuArchive);
			pm.MenuCommands.Add(mnuAddFav);
			pm.MenuCommands.Add(mnuDelete);
			pm.MenuCommands.Add(mnuDiarise);
			pm.MenuCommands.Add(mnuNewCase);
			pm.MenuCommands.Add(mnuReactivate);
			pm.MenuCommands.Add(mnuRefresh);
			pm.MenuCommands.Add(mnuDelFav);

			this.SetStyle(ControlStyles.DoubleBuffer, true);
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
					foreach (IndexNode x in IndexNodes.Values)
					{
						ClearAll(x.Nodes);
						x.CleanUp();
					}
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(CaseTreeViewEx));
			this.TreeImages = new System.Windows.Forms.ImageList(this.components);
			this.scrollTimer = new System.Windows.Forms.Timer(this.components);
			this.mnuAddFav = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuDelFav = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuDiarise = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuArchive = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuNewCase = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuDelete = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuReactivate = new Crownwood.Magic.Menus.MenuCommand();
			this.mnuRefresh = new Crownwood.Magic.Menus.MenuCommand();
			// 
			// TreeImages
			// 
			this.TreeImages.ColorDepth = System.Windows.Forms.ColorDepth.Depth8Bit;
			this.TreeImages.ImageSize = new System.Drawing.Size(16, 16);
			this.TreeImages.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("TreeImages.ImageStream")));
			this.TreeImages.TransparentColor = System.Drawing.Color.Transparent;
			// 
			// scrollTimer
			// 
			this.scrollTimer.Interval = 333;
			this.scrollTimer.Tick += new System.EventHandler(this.scrollTimer_Tick);
			// 
			// mnuAddFav
			// 
			this.mnuAddFav.Description = "Add to favourites";
			this.mnuAddFav.Text = "&Add to favourites";
			this.mnuAddFav.Click += new System.EventHandler(this.mnuAddFav_Click);
			// 
			// mnuDelFav
			// 
			this.mnuDelFav.Description = "Remove from favourites";
			this.mnuDelFav.Text = "&Remove from favourites";
			this.mnuDelFav.Click += new System.EventHandler(this.mnuDelFav_Click);
			// 
			// mnuDiarise
			// 
			this.mnuDiarise.Description = "Diarise this case";
			this.mnuDiarise.Text = "&Diarise";
			this.mnuDiarise.Click += new System.EventHandler(this.mnuDiarise_Click);
			// 
			// mnuArchive
			// 
			this.mnuArchive.Description = "Archive this case";
			this.mnuArchive.Text = "A&rchive Case";
			this.mnuArchive.Click += new System.EventHandler(this.mnuArchive_Click);
			// 
			// mnuNewCase
			// 
			this.mnuNewCase.Description = "New case";
			this.mnuNewCase.Text = "&New Case";
			this.mnuNewCase.Click += new System.EventHandler(this.mnuNewCase_Click);
			// 
			// mnuDelete
			// 
			this.mnuDelete.Description = "Delete this case";
			this.mnuDelete.Text = "&Delete Case";
			this.mnuDelete.Click += new System.EventHandler(this.mnuDelete_Click);
			// 
			// mnuReactivate
			// 
			this.mnuReactivate.Description = "Reactivate this case";
			this.mnuReactivate.Text = "&Re-activate Case";
			this.mnuReactivate.Click += new System.EventHandler(this.mnuReactivate_Click);
			// 
			// mnuRefresh
			// 
			this.mnuRefresh.Description = "Refresh this list";
			this.mnuRefresh.Text = "&Refresh";
			this.mnuRefresh.Click += new System.EventHandler(this.mnuRefresh_Click);

		}
		#endregion

		public void FillDefaultDebtTree()
		{
			this.BeginUpdate();
			if (IndexNodes==null)
			{
				IndexNodes = new Hashtable();
			}
			else
			{
				foreach (DictionaryEntry x in IndexNodes)
				{
					((IndexNode)x.Value).CleanUp();
				}
				ClearAll(this.Nodes);
				IndexNodes.Clear();
			}				
			TreeNode RootNode = new TreeNode("Arrears Manager Index", 0,0);
			
			IndexNodes.Add("AC", RootNode.Nodes[RootNode.Nodes.Add(new IndexNode("Account Number", 7,8))]);
			IndexNodes.Add("AD", RootNode.Nodes[RootNode.Nodes.Add(new CRReverseIndexNode("Address", 7,8))]);
			IndexNodes.Add("CN", RootNode.Nodes[RootNode.Nodes.Add(new SimpleIndexNode("Customer Name", 7,8))]);
			IndexNodes.Add("DG", RootNode.Nodes[RootNode.Nodes.Add(new SimpleIndexNode("Degree", 7,8))]);
			IndexNodes.Add("PD", RootNode.Nodes[RootNode.Nodes.Add(new DateIndexNode("Possession Date", 7,8))]);
			IndexNodes.Add("PC", RootNode.Nodes[RootNode.Nodes.Add(new SpaceIndexNode("Post Code", 7,8))]);
			IndexNodes.Add("AR", RootNode.Nodes[RootNode.Nodes.Add(new IndexNode("Archived Cases", 11,12))]);
			IndexNodes.Add("FV", RootNode.Nodes[RootNode.Nodes.Add(new IndexNode("Favourites", 13,14))]);

			RootNode.Expand();

			string sql = null;
			string enumerator = null;

			for (int i = 0; i < IndexNodes.Count; i++)
			{
				switch (i)
				{
					case 0:
						enumerator = "AC";
						sql = "Select AccountNumber, ID from Cases Where Archived = false order by AccountNumber";
						break;
					case 1:
						enumerator = "AD";
						sql = "Select AccountNumber, PropertyAddress, ID From Cases Where Archived = false";
						break;
					case 2:
						enumerator = "CN";
						sql = "Select Title & ' ' & Initials & ': ' & AccountNumber, Surname, cases.ID From Cases Left Join People on Cases.ID = People.CaseID Where Archived = false order by Surname, Initials";
						break;
					case 3:
						enumerator = "DG";
						sql = "Select AccountNumber, Degree, ID from Cases Where Archived = false order by Degree";
						break;
					case 4:
						enumerator = "PD";
						sql = "Select AccountNumber, PossessionDate, ID from Cases Where Archived = false Order By PossessionDate";
						break;
					case 5:
						enumerator = "PC";
						sql = "Select AccountNumber, PropertyPostCode, ID from Cases Where Archived = false Order By PropertyPostCode";
						break;
					case 6:
						enumerator = "AR";
						sql = "Select AccountNumber, ID From Cases Where Archived = true Order By AccountNumber";
						break;
					case 7:
						enumerator = "FV";
						sql = "Select AccountNumber, cases.ID From Cases, userfavourites Where Archived = false and CaseID = Cases.ID And UserID = " + CSession.User.UserID.ToString() + " order By AccountNumber";
						break;
				}

				using (DataTable myDT = cMyDB.GetDataTable(sql))
				{
					FillIndexNode((IndexNode)IndexNodes[enumerator], myDT);
				}
			}
			this.Nodes.Add(RootNode);
			this.EndUpdate();
			CSession.CaseSelected += new CaseEventHandler(CaseSelected_EventHandler);
		}
				
		private void CaseSelected_EventHandler(object sender, CaseEventArgs e)
		{
			if (this.SelectedNode is CaseTreeNode)
			{
				if (((CaseTreeNode)this.SelectedNode).CaseID != e.CaseID)
				{
					this.ShowCase(e.CaseID, true);
				}
			}
			else
			{
				this.ShowCase(e.CaseID, true);
			}
		}

		private void RefreshFavouritesNode()
		{
			ClearAll(((IndexNode)IndexNodes["FV"]).Nodes);
			FillFavouritesNode();
		}
		private void FillFavouritesNode()
		{
			string enumerator = "FV";
			string sql = "Select AccountNumber, cases.ID From Cases, userfavourites Where CaseID = Cases.ID And UserID = " + CSession.User.UserID.ToString();
			using (DataTable myDT = cMyDB.GetDataTable(sql))
			{
				FillIndexNode((IndexNode)IndexNodes[enumerator], myDT);
			}
		}
		private void FillIndexNode(IndexNode catNode, DataTable dt)
		{
			switch (dt.Columns.Count)
			{
				case 2:
					foreach (DataRow dr in dt.Rows)
					{
						if (dr[0].Equals(DBNull.Value))
							catNode.AddNode("A/C Not Specified " + dr[1].ToString(),dr[0],(Int32)dr[1]);
						else
                            catNode.AddNode(dr[0].ToString(),dr[0],(Int32)dr[1]);
					}
					break;
				case 3:
					foreach (DataRow dr in dt.Rows)
					{
						if (dr[0].Equals(DBNull.Value))
							catNode.AddNode("A/C Not Specified " + dr[2].ToString(),dr[1],(Int32)dr[2]);
						else
							catNode.AddNode(dr[0].ToString(),dr[1],(Int32)dr[2]);
					}
					break;
			}
			catNode.LastDataLoad = DateTime.Now;
			catNode.hasData = true;
		}
		public void AddFavourite(Int32 CaseID)
		{
			OleDbCommand cmd = new OleDbCommand("Select ID From Cases Where ID = ?");
			OleDbParameter parID = new OleDbParameter("id", OleDbType.Integer);
			parID.Value = CaseID;
			cmd.Parameters.Add(parID);

			if (!cMyDB.IsEmpty(cmd))
			{
				cmd.Parameters.Clear();
				cmd.CommandText = "Insert Into UserFavourites (CaseID, UserID, DateAdded) Values (?, ?, ?)";
				OleDbParameter parUser = new OleDbParameter("user", OleDbType.Integer);
				parUser.Value = CSession.User.UserID;
				OleDbParameter parDateAdded = new OleDbParameter("dateAdded", OleDbType.Date);
				parDateAdded.Value = DateTime.Now;

				cmd.Parameters.Add(parID);
				cmd.Parameters.Add(parUser);
				cmd.Parameters.Add(parDateAdded);

				cMyDB.Execute(cmd);
			}
			cmd.Dispose();
			RefreshFavouritesNode();
		}
		public void DeleteFavourite(Int32 CaseID)
		{
			OleDbCommand cmd = new OleDbCommand("Select ID From UserFavourites Where CaseID = ? And UserID = ?");
			OleDbParameter parID = new OleDbParameter("caseid", OleDbType.Integer);
			parID.Value = CaseID;
			OleDbParameter parUser = new OleDbParameter("userid", OleDbType.Integer);
			parUser.Value = CSession.User.UserID;
			
			cmd.Parameters.Add(parID);
			cmd.Parameters.Add(parUser);				

			if (!cMyDB.IsEmpty(cmd))
			{
				cmd.CommandText = "Delete From UserFavourites Where CaseID = ? And UserID = ?";
				cMyDB.Execute(cmd);
			}
			cmd.Dispose();
			RefreshFavouritesNode();
		}
		private void ClearAll(TreeNodeCollection nc)
		{
			foreach (TreeNode x in nc)
			{
				if (x.Nodes.Count > 0)
				{
					ClearAll(x.Nodes);
				}
			}
			nc.Clear();
		}
		public void ShowCase(Int32 CaseID)
		{
			ShowCase(CaseID, false);
		}

		private void ShowCase(Int32 CaseID, bool SupressEvents)
		{
			mSupressEvents = SupressEvents;
			this.SelectedNode = FindNode("AC", CaseID);
			if (this.SelectedNode != null)
				this.SelectedNode.EnsureVisible();
			else
			{
				this.SelectedNode = FindNode("AR", CaseID);
				if (this.SelectedNode != null)
					this.SelectedNode.EnsureVisible();
			}
			mSupressEvents = false;
		}
		public TreeNode FindNode(string LookIn, Int32 CaseID)
		{
			return ((IndexNode)IndexNodes[LookIn]).Indexer.GetNode(CaseID);
		}

		protected override void OnAfterSelect(System.Windows.Forms.TreeViewEventArgs e)
		{			
			base.OnAfterSelect(e);
			if (this.SelectedNode is CaseTreeNode)
			{	 
				if (!mSupressEvents)
					CSession.SelectCase(((CaseTreeNode)this.SelectedNode).CaseID);
				BeginUpdate();
				HideSelection = false;
				EndUpdate();
			}
			else
			{
				BeginUpdate();
				HideSelection = true;
				EndUpdate();
			}
		}

		protected override void OnMouseDown(System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left)
			{
				if (this.GetNodeAt(e.X,e.Y)!=null )
				{
					if (this.GetNodeAt(e.X,e.Y) is CaseTreeNode)
					{
						DragCaseNode = (CaseTreeNode)this.GetNodeAt(e.X,e.Y);
						scrollTimer.Enabled = true;
						if(this.DoDragDrop(DragCaseNode.CaseID.ToString(), DragDropEffects.Copy | DragDropEffects.Move) == DragDropEffects.None)
						{
							this.SelectedNode = this.GetNodeAt(e.X,e.Y);
							if (!this.Focused)
								this.Focus();
						}
						scrollTimer.Enabled = false;
					}
				}
			}
			base.OnMouseDown(e);
		}

		protected override void OnMouseUp(System.Windows.Forms.MouseEventArgs e)
		{
			if (e.Button == MouseButtons.Right)
			{
				if (this.GetNodeAt(e.X,e.Y) != null)
				{
					if (this.GetNodeAt(e.X,e.Y) is CaseTreeNode)
					{
						CaseTreeNode TempCN = (CaseTreeNode)this.GetNodeAt(e.X,e.Y);
						
						if (CSession.ArchiveCases)
						{
							if (TempCN.Parent.Text == "Archived Cases")
							{
								mnuArchive.Visible = false;
								mnuReactivate.Visible = true;
								mnuReactivate.UserData = TempCN.CaseID;
							}
							else
							{
								mnuReactivate.Visible = false;
								mnuArchive.Visible = true;
								mnuArchive.UserData = TempCN.CaseID;
							}
						}
						else
						{
							mnuReactivate.Visible = false;
							mnuArchive.Visible = false;
						}

						if (CSession.DeleteCases)
						{
							mnuDelete.Visible = true;
							mnuDelete.UserData = TempCN.CaseID;
						}
						else
							mnuDelete.Visible = false;

						if (CSession.AddCases)
							mnuNewCase.Visible = true;		
						else
							mnuNewCase.Visible = false;
					

						if (TempCN.Parent.Text == "Favourites")
						{
							mnuAddFav.Visible = false;
							mnuDelFav.Visible = true;
							mnuDelFav.UserData = TempCN.CaseID;
							mnuDiarise.Visible = true;
							mnuDiarise.UserData = TempCN.CaseID;
						}
						else
						{
							mnuAddFav.Visible = true;							
							mnuAddFav.UserData = TempCN.CaseID;
							mnuDelFav.Visible = false;
							mnuDiarise.Visible = true;
							mnuDiarise.UserData = TempCN.CaseID;
						}
					}
					else
					{
						if (CSession.AddCases)
							mnuNewCase.Visible = true;		
						else
							mnuNewCase.Visible = false;
						mnuRefresh.Visible = true;
						mnuAddFav.Visible = false;
						mnuDelFav.Visible = false;
						mnuDiarise.Visible = false;
						mnuDelete.Visible = false;
						mnuArchive.Visible = false;
						mnuReactivate.Visible = false;
					}
				}
				else
				{
					if (CSession.AddCases)
						mnuNewCase.Visible = true;		
					else
						mnuNewCase.Visible = false;
					mnuRefresh.Visible = true;
					mnuAddFav.Visible = false;
					mnuDelFav.Visible = false;
					mnuDiarise.Visible = false;
					mnuDelete.Visible = false;
					mnuArchive.Visible = false;
					mnuReactivate.Visible = false;
				}
				pm.TrackPopup(this.PointToScreen(new Point(e.X,e.Y)));
			}
			
			base.OnMouseUp(e);
		}

		protected override void OnDragDrop(System.Windows.Forms.DragEventArgs drgevent)
		{
			try
			{
				if(IndexNodes["FV"] == this.GetNodeAt(this.PointToClient(new Point(drgevent.X, drgevent.Y))))
					AddFavourite(Convert.ToInt32(drgevent.Data.GetData(DataFormats.Text)));
				else if (IndexNodes["AR"] == this.GetNodeAt(this.PointToClient(new Point(drgevent.X, drgevent.Y))))
				{
					if (CSession.ArchiveCases)
					{
						if (ArchiveCase!=null)
							ArchiveCase(this, new CaseEventArgs(Convert.ToInt32(drgevent.Data.GetData(DataFormats.Text))));
					}
				}
			}
			catch {}
		}

		protected override void OnDragLeave(System.EventArgs e)
		{
			DragCaseNode = null;
			base.OnDragLeave(e);
		}

		protected override void OnDragOver(System.Windows.Forms.DragEventArgs drgevent)
		{
			if(IndexNodes["FV"] == this.GetNodeAt(this.PointToClient(new Point(drgevent.X, drgevent.Y))))
				drgevent.Effect = DragDropEffects.Copy;
			else if (IndexNodes["AR"] == this.GetNodeAt(this.PointToClient(new Point(drgevent.X, drgevent.Y))))
			{
				if (CSession.ArchiveCases)
					drgevent.Effect = DragDropEffects.Move;
			}
			else
				drgevent.Effect = DragDropEffects.None;
			base.OnDragOver(drgevent);
		}

		private void scrollTimer_Tick(object sender, System.EventArgs e)
		{
			int CurY = PointToClient(MousePosition).Y;
			if (CurY > Height - 25 && CurY < Height)
			{
				CUtil.ScrollDown(Handle);
			}
			else if (CurY > 0 && CurY < 25)
			{
				CUtil.ScrollUp(Handle);
			}
		}

		private void mnuAddFav_Click(object sender, System.EventArgs e)
		{
			try
			{
				AddFavourite((Int32)mnuAddFav.UserData);
			}
			catch {}
		}

		private void mnuDelFav_Click(object sender, System.EventArgs e)
		{
			DeleteFavourite((Int32)mnuDelFav.UserData);
		}

		private void mnuDiarise_Click(object sender, System.EventArgs e)
		{
			CDiary.Diarise((Int32)mnuDiarise.UserData);
		}

		private void mnuArchive_Click(object sender, System.EventArgs e)
		{
			if (ArchiveCase!=null)
				ArchiveCase(this, new CaseEventArgs((Int32)mnuArchive.UserData));
		}
		
		private void mnuDelete_Click(object sender, System.EventArgs e)
		{
			if (DeleteCase!=null)
				DeleteCase(this, new CaseEventArgs((Int32)mnuDelete.UserData));
		}

		private void mnuReactivate_Click(object sender, System.EventArgs e)
		{
			if (ReactivateCase!=null)
				ReactivateCase(this, new CaseEventArgs((Int32)mnuReactivate.UserData));
		}

		private void mnuRefresh_Click(object sender, System.EventArgs e)
		{
			FillDefaultDebtTree();
		}

		private void mnuNewCase_Click(object sender, System.EventArgs e)
		{
			if (NewCase!=null)
				NewCase(this, new CaseEventArgs(0));
		}
	}
	public class CRReverseIndexNode : IndexNode
	{
		public CRReverseIndexNode(string text, int image, int selectedImage)	
			: base(text, image, selectedImage)	{}

		protected override TreeNode PlaceInHierachy(object orderData)
		{
			if (orderData.Equals(DBNull.Value))
			{
				return this.NullNode;
			}
			string[] tokens = ((String)orderData).Split(new Char[] {'\n', '\r'});
			int strCount = tokens.Length;
			TreeNode nodeLevel;
			bool levelFound = false;
			nodeLevel = (TreeNode) this;

			for (int i = strCount; i>0; i--)
			{
				if (tokens[i-1]=="")
					continue;
				levelFound = false;
				foreach(TreeNode n in nodeLevel.Nodes)
				{
					if (n.Text == tokens[i-1])
					{
						nodeLevel = n;
						levelFound = true;
						break;
					}
				}
				if (!levelFound)
				{
					nodeLevel = nodeLevel.Nodes.Add(tokens[i-1]);
					nodeLevel.ImageIndex = this.ImageIndex;
					nodeLevel.SelectedImageIndex = this.SelectedImageIndex;
				}
			}
			return (TreeNode) nodeLevel;
		}
	}
	public class SpaceIndexNode : IndexNode
	{
		public SpaceIndexNode(string text, int image, int selectedImage)	
			: base(text, image, selectedImage)	{}

		protected override TreeNode PlaceInHierachy(object orderData)
		{
			if (orderData.Equals(DBNull.Value))
			{
				return this.NullNode;
			}
			string[] tokens = (Convert.ToString(orderData)).Split(new char[]{' '});
			int strCount = tokens.Length;
			TreeNode nodeLevel;
			bool levelFound = false;
			nodeLevel = (TreeNode) this;

			for (int i = 0; i<strCount; i++)
			{
				levelFound = false;
				foreach(TreeNode n in nodeLevel.Nodes)
				{
					if (n.Text == tokens[i])
					{
						nodeLevel = n;
						levelFound = true;
						break;
					}
				}
				if (!levelFound)
				{
					nodeLevel = nodeLevel.Nodes.Add(tokens[i]);
					nodeLevel.ImageIndex = this.ImageIndex;
					nodeLevel.SelectedImageIndex = this.SelectedImageIndex;
				}
			}
			return (TreeNode) nodeLevel;
		}
	}
	public class DateIndexNode : IndexNode
	{
		public DateIndexNode(string text, int image, int selectedImage)	
			: base(text, image, selectedImage)	{}

		protected override TreeNode PlaceInHierachy(object orderData)
		{
			if (orderData.Equals(DBNull.Value))
			{
				return this.NullNode;
			}
			TreeNode nodeLevel;
			DateTime convData = Convert.ToDateTime(orderData);
			bool levelFound = false;
			nodeLevel = (TreeNode) this;
			string TempData = null;

			for (int i = 1; i<4; i++)
			{
				switch (i)
				{
					case 1:
						TempData = Convert.ToString(convData.Year);
						break;
					case 2:
						TempData = String.Format("{0:MMMM}",convData);
						break;
					case 3:
						TempData = Convert.ToString(convData.Day);
						break;
				}
                
				foreach(TreeNode n in nodeLevel.Nodes)
				{
					if (n.Text == TempData)
					{
						nodeLevel = n;
						levelFound = true;
						break;
					}
				}
				if (!levelFound)
				{
					nodeLevel = nodeLevel.Nodes.Add(TempData);
					nodeLevel.ImageIndex = this.ImageIndex;
					nodeLevel.SelectedImageIndex = this.SelectedImageIndex;
				}
			}
			return (TreeNode) nodeLevel;
		}
	}
	public class SimpleIndexNode : IndexNode
	{
		public SimpleIndexNode(string text, int image, int selectedImage)	
			: base(text, image, selectedImage)	{}

		protected override TreeNode PlaceInHierachy(object orderData)
		{
			if (orderData.Equals(DBNull.Value))
			{
				return this.NullNode;
			}			
			TreeNode nodeLevel;
			string convData = Convert.ToString(orderData);
			bool levelFound = false;
			nodeLevel = (TreeNode) this;

			foreach(TreeNode n in nodeLevel.Nodes)
			{
				if (n.Text == convData)
				{
					nodeLevel = n;
					levelFound = true;
					break;
				}
			}
			if (!levelFound)
			{
				nodeLevel = nodeLevel.Nodes.Add(convData);
				nodeLevel.ImageIndex = this.ImageIndex;
				nodeLevel.SelectedImageIndex = this.SelectedImageIndex;
			}
			return (TreeNode) nodeLevel;
		}
	}
	public class IndexNode : TreeNode
	{
		public CaseNodeIndexer Indexer;
		public DateTime LastDataLoad;
		public bool hasData = false;
		protected TreeNode mNullNode = null;

		public IndexNode()
		{
			Indexer = new CaseNodeIndexer();
		}
		public IndexNode(string text, int image, int selectedImage)
		{
			this.Text = text;
			this.ImageIndex = image;
			this.SelectedImageIndex = selectedImage;
			Indexer = new CaseNodeIndexer();
		}
		virtual protected TreeNode PlaceInHierachy(object orderData) 
		{
			return (TreeNode) this;
		}		
		public void AddNode(string Text, object orderData, Int32 CaseID)
		{
			TreeNode n = PlaceInHierachy(orderData);
			CaseTreeNode newNode = new CaseTreeNode(Text, 15,15,CaseID);
			n.Nodes.Add(newNode);
			Indexer.AddNode(CaseID, newNode);		
		}
		public void CleanUp()
		{
			Indexer.CleanUp();
			Indexer = null;
		}
		public TreeNode NullNode
		{
			get
			{
				if (mNullNode!=null)
					return mNullNode;
				else
				{
					mNullNode = new TreeNode("Not Specified", this.ImageIndex, this.SelectedImageIndex);
					this.Nodes.Add(mNullNode);
					return mNullNode;
				}
			}
		}		
	}
	public class CaseTreeNode : TreeNode
	{
		private Int32 mCaseID = 0;

		public CaseTreeNode(string text, int image, int selectedImage, Int32 caseID)
		{
			this.Text = text;
			this.ImageIndex = image;
			this.SelectedImageIndex = selectedImage;
			mCaseID = caseID;
		}

		public Int32 CaseID
		{
			get 
			{
				return mCaseID;
			}
		}
	}
	public class CaseNodeIndexer
	{
		private System.Collections.Hashtable htList = null;

		public CaseNodeIndexer()
		{
			htList = new Hashtable();
		}
		public void AddNode(Int32 CaseID, CaseTreeNode node)
		{
			if (!htList.Contains(CaseID))
				htList.Add(CaseID, node);
		}
		public CaseTreeNode GetCaseNode(Int32 CaseID)
		{
			return (CaseTreeNode) htList[CaseID];
		}
		public TreeNode GetNode(Int32 CaseID)
		{
			return (TreeNode) htList[CaseID];
		}
		public bool Contains(Int32 CaseID)
		{
			return htList.Contains(CaseID);
		}
		public void CleanUp()
		{
			htList.Clear();
			htList = null;
		}
	}
}
