using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Data.OleDb;
using System.Windows.Forms;

namespace DebtManagerTest.LookupComboObjects
{
	/// <summary>
	/// Lookup box which uses the LookupDefinitions table to obtain its
	/// data, based on the value of LookupID.
	/// 
	/// NOTE: Cannot bind to Selected Value from other forms with this
	/// implementation. Unless Databindings are cleared first.
	/// 
	/// So far only been tested as dropdownlist.
	/// </summary>
	public class LookupCombo : System.Windows.Forms.ComboBox
	{
		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private string mLookupID = "";
		private LookupConfig lc;
		private bool hasConfig = false;
		private DataTable dt;
		private System.Windows.Forms.ContextMenu mnu1;
		private System.Windows.Forms.MenuItem mnuAdd;
		private System.Windows.Forms.MenuItem mnuDelete;
		private System.Windows.Forms.MenuItem mnuEdit;
		private System.Windows.Forms.MenuItem mnuNull;
		private System.Windows.Forms.MenuItem mnus1;
		//Used by the change selection overrides to store selected object
		// in case a new one isn't entered and we need to reselect the old one
		private int mCurrentIndex = -1;
		private string mCurrentSelection;
		private bool mExtraRefreshes = true;
		private bool mAllowUpdates = true;
		//Combo box behavior seems less unpredictable when bound to 
		//an object variable.
		private object mComboValue = null;
		private bool mDataBound = true;
		private bool mNullable = true;

		public bool Nullable
		{
			get 
			{
				return mNullable;
			}
			set
			{
				mNullable = value;
			}
		}
		public bool DataBound
		{
			get 
			{
				return mDataBound;
			}
			set
			{
				mDataBound = value;
			}
		}
		public object ComboValue
		{
			get 
			{
				return mComboValue;
			}
			set
			{
				mComboValue = value;
			}
		}
		public LookupCombo()
		{
			InitializeComponent();
			DropDownStyle = ComboBoxStyle.DropDownList;
		}
		private void GetLookupConfig()
		{
			lc = LookupDataManager.GetConfig(mLookupID);
			hasConfig = true;
		}
		public void RefreshDataSource(bool reSelectCurrent)
		{
			RefreshDataSource();
			if (reSelectCurrent)
				SelectedIndex = FindString(mCurrentSelection);
		}
		public void RefreshDataSource()
		{
			if (dt != null) 
			{
				if (mCurrentIndex != -1)
				{
					mCurrentSelection = this.GetItemText(this.Items[mCurrentIndex]);
					mCurrentIndex = -1;
				}
				SelectedIndex = -1;
				dt.Dispose();
			}
			dt = cMyDB.GetDataTable(lc.SQL);
			if (lc.Updateable && AllowUpdates && (CSession.User.UserLevel >= lc.SecurityLevel))
			{
				// <new value> added to allow users to add new items to the list 
				// simply by selecting from the list.
				DataRow dr = dt.NewRow();
				if (lc.Composite!="")
					dr["CompositeValue"] = "<new value>";
				else
					dr["Description"] = "<new value>";
				if (lc.HasIDField)
				{
					if (dt.Columns["ID"].DataType == typeof(String))
						dr["ID"] = "-0";
					else
						dr["ID"] = -32446655;
				}
				dt.Rows.Add(dr);
				dr = null;
			}
			DataSource = dt;
			if (lc.Composite != "")
				DisplayMember = "CompositeValue";      // used for cases where either id or description in the combo box would be insufficient
			else
				DisplayMember = "Description";

			if (lc.LookupType > 1)
				ValueMember = "ID";
			else
				ValueMember = "Description";
			SelectedIndex = -1;
		}

		public void FillLookupCombo()
		{
			if (!hasConfig)
				GetLookupConfig();
			
			RefreshDataSource();
		}
		public string LookupID
		{
			get
			{
				return mLookupID;
			}
			set 
			{
				mLookupID = value;
				hasConfig = false;
				if (!DesignMode)
				{
					if (mLookupID != "")
					{
						if (!DataBound)
						{
							this.DataBindings.Clear();
							ComboValue = null;
							this.DataBindings.Add("SelectedValue",this,"ComboValue");
						}
						FillLookupCombo();
					}
					SelectedIndex = -1;
				}
			}
		}
		public bool AllowUpdates
		{
			get
			{
				return mAllowUpdates;
			}
			set
			{
				mAllowUpdates = value;
			}
		}
		public bool ExtraRefreshes
		{
			get 
			{
				return mExtraRefreshes;
			}
			set
			{
				mExtraRefreshes = value;
			}
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
				if (lc!=null)
					lc.CleanUp();
				if (dt!=null)
					dt.Dispose();
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.mnu1 = new System.Windows.Forms.ContextMenu();
			this.mnuAdd = new System.Windows.Forms.MenuItem();
			this.mnuDelete = new System.Windows.Forms.MenuItem();
			this.mnuEdit = new System.Windows.Forms.MenuItem();
			this.mnuNull = new System.Windows.Forms.MenuItem();
			this.mnus1 = new System.Windows.Forms.MenuItem();
			// 
			// mnu1
			// 
			this.mnu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {   this.mnuAdd,
																				 this.mnuNull,
																				 this.mnus1,
																				 this.mnuEdit,
																				 this.mnuDelete});
			this.mnu1.Popup += new System.EventHandler(this.mnu1_Popup_1);
			// 
			// mnuAdd
			// 
			this.mnuAdd.DefaultItem = true;
			this.mnuAdd.Index = 0;
			this.mnuAdd.Text = "&Add New Item";
			this.mnuAdd.Click += new System.EventHandler(this.mnuAdd_Click);
			// 
			// mnuNull
			// 
			this.mnuNull.Index = 1;
			this.mnuNull.Text = "Deselect";
			this.mnuNull.Click += new System.EventHandler(this.mnuNull_Click);
			// 
			// mnus1
			// 
			this.mnus1.Index = 2;
			this.mnus1.Text = "-";
			// 
			// mnuEdit
			// 
			this.mnuEdit.Index = 3;
			this.mnuEdit.Text = "&Edit Selected Item";
			this.mnuEdit.Click += new System.EventHandler(this.mnuEdit_Click);
			// 
			// mnuDelete
			// 
			this.mnuDelete.Index = 4;
			this.mnuDelete.Text = "&Delete Selected Item";
			this.mnuDelete.Click += new System.EventHandler(this.mnuDelete_Click);
			// 
			// LookupCombo
			// 
			this.ContextMenu = this.mnu1;

		}
		#endregion

		public string addNewItem()
		{
			bool release = false;
			bool ItemAdded = false; 
			LookupAddAndEditForm frm = new LookupAddAndEditForm();
			lc.FormatForm(frm);
			while (!release)
			{
				release = true; //used to control exit from the while loop. this is so it will be repeated if the user makes an error on the form
				DialogResult d = frm.ShowDialog();
				if (d == DialogResult.OK)
				{
					// check if input is valid
					if (!lc.ValidateDescription(frm.txtDescription.Text)) 
					{
						MessageBox.Show("The data you have entered is not valid. Please ensure all fields have been completed.","Input Error");
						release = false;
						frm.txtAdditional.Select();
					}
					if (lc.GetAdditionalValue(frm) == "" && lc.ExtraField != "") //TODO: Update this for combo box validate routine
					{
						MessageBox.Show("The data you have entered is not valid. Please ensure all fields have been completed.","Input Error");
						release = false;
					}
					if (!lc.ValidateID(frm.txtID.Text))
					{
						MessageBox.Show("The data you have entered is not valid. Please ensure all fields have been completed.","Input Error");
						frm.txtID.Select();
						release = false;
					}
					if (release)
					{
						try 
						{
							//make sure the new value doesn't already exist
							object retval = cMyDB.GetResult(lc.GetDuplicateCheckSQL(frm.txtID.Text, frm.txtDescription.Text));
							if (retval == null)
							{
								try 
								{
									//add to the database
									cMyDB.Execute(lc.GetInsertSQL(frm.txtID.Text,frm.txtDescription.Text,lc.GetAdditionalValue(frm)));
									ItemAdded = true;
								}
								catch (Exception e)
								{
									MessageBox.Show("An error occured trying to add this data to the database. Details: " + e.Message, "Error");
									release = false;
								}
							}
							else //to catch duplicate values
							{
								MessageBox.Show("The data you have entered cannot be accepted as it would conflict with an existing item in this list.", "Input Error");
								release = false;
							}
						}
						catch (Exception e) //to catch unexpected exceptions
						{
							MessageBox.Show("An error has occured : " + e.Message, "Unexpected Error");
						}
					}
				}
			}
			string retStr = frm.txtDescription.Text;
			frm.Dispose();
			if (ItemAdded)
				return retStr;
			else
				return null;
		}

		public string updateItem()
		{			
			bool release = false;
			bool ItemUpdated = false; 
			bool UpdateCancelled = false;
			LookupAddAndEditForm frm = new LookupAddAndEditForm();
			lc.FormatForm(frm, true, dt);
			try
			{
				frm.BindingContext[dt].Position = SelectedIndex;
			}
			catch (Exception ex)
			{
				MessageBox.Show("Error opening the editor. " + ex.Message,"System Error");
				return null;
			}

			while (!release)
			{
				release = true; //used to control exit from the while loop. this is so it will be repeated if the user makes an error on the form
				DialogResult d = frm.ShowDialog();
				if (d == DialogResult.OK)
				{
					// check if input is valid
					if (!lc.ValidateDescription(frm.txtDescription.Text))
					{
						MessageBox.Show("The data you have entered is not valid. Please ensure all fields have been completed.","Input Error");
						frm.txtDescription.Select();
						release = false;
					}
					if (!lc.ValidateID(frm.txtID.Text))
					{
						MessageBox.Show("The data you have entered is not valid. Please ensure all fields have been completed.","Input Error");
						frm.txtID.Select();
						release = false;
					}
					if (lc.GetAdditionalValue(frm)=="" && lc.ExtraField!="")
					{
						MessageBox.Show("The data you have entered is not valid. Please ensure all fields have been completed.","Input Error");
						release = false;
					}
					if (release)
					{
						try 
						{
							//make sure the new value doesn't already exist
							object retval = null;
							if (dt.Rows[SelectedIndex][lc.DuplicateKeyField,DataRowVersion.Original] != dt.Rows[SelectedIndex][lc.DuplicateKeyField,DataRowVersion.Current])
								retval = cMyDB.GetResult(lc.GetDuplicateCheckSQL(frm.txtID.Text, frm.txtDescription.Text));
							if (retval == null)
							{
								try 
								{
									//add to the database
									if (lc.HasIDField)
										retval = cMyDB.GetResult(lc.GetCheckUpdateDateSQL(
											Convert.ToString(dt.Rows[SelectedIndex]["ID",DataRowVersion.Original])
											,(string)dt.Rows[SelectedIndex]["Description",DataRowVersion.Original]));
									else
										retval = cMyDB.GetResult(lc.GetCheckUpdateDateSQL(
											""
											,(string)dt.Rows[SelectedIndex]["Description",DataRowVersion.Original]));

									if (retval == null)
										throw new Exception("The list item you are trying to edit has been modified by another user and can not be identified. You cannot edit this item. Review the updated drop down list contents, if you still want to change one of the entries you will be able to.");

									if (!retval.Equals(dt.Rows[SelectedIndex]["LastUpdated"]))
									{
										DataTable current;
										if (lc.HasIDField)
											current = cMyDB.GetDataTable(lc.GetCurrentValuesSQL(
												Convert.ToString(dt.Rows[SelectedIndex]["ID",DataRowVersion.Original])
												,(string)dt.Rows[SelectedIndex]["Description",DataRowVersion.Original]));
										else
											current = cMyDB.GetDataTable(lc.GetCurrentValuesSQL(
												""
												,(string)dt.Rows[SelectedIndex]["Description",DataRowVersion.Original]));
								
										if (current.Rows.Count == 0)
											throw new Exception("The list item you are trying to edit has been modified by another user and can not be identified. You cannot edit this item. Review the updated drop down list contents, if you still want to change one of the entries you will be able to.");
										
										RecordConflict rcFrm = new RecordConflict();

										rcFrm.Compare(current.Rows[0], dt.Rows[SelectedIndex]);
										
										rcFrm.ShowDialog();
										
										if (rcFrm.DialogResult != DialogResult.OK)
											UpdateCancelled = true;
										
										rcFrm.Dispose();
										current.Dispose();
									}

									string Usql;
									if (!UpdateCancelled)
									{
										if (lc.HasIDField)
											Usql = lc.GetUpdateSQL(
												Convert.ToString(dt.Rows[SelectedIndex]["ID",DataRowVersion.Original])
												,frm.txtID.Text
												,(string) dt.Rows[SelectedIndex]["Description",DataRowVersion.Original]
												,frm.txtDescription.Text
												,lc.GetAdditionalValue(frm));
										else
											Usql = lc.GetUpdateSQL(
												""
												,frm.txtID.Text
												,(string) dt.Rows[SelectedIndex]["Description",DataRowVersion.Original]
												,frm.txtDescription.Text
												,lc.GetAdditionalValue(frm));

										cMyDB.Execute(Usql);
										ItemUpdated = true;
									}
								}
								catch (Exception ex)
								{
									MessageBox.Show("This update could not be performed. " + ex.Message, "Error");
									release = false;
								}
							}
							else //to catch duplicate values
							{
								MessageBox.Show("The data you have entered cannot be accepted as it would conflict with an existing item in this list.", "Input Error");
								release = false;
							}
						}
						catch (Exception ex) //to catch unexpected exceptions
						{
							MessageBox.Show("An error has occured : " + ex.Message, "Unexpected Error");
						}
					}
				}
			}
			string retStr = frm.txtDescription.Text;
			frm.Dispose();
			if (ItemUpdated)
				return retStr;
			else
				return null;
		}

		private bool deleteItem()
		{
			bool ItemDeleted = false; 
			bool DeleteCancelled = false;
			int noOfDependents = lc.DependentsCount;
			bool hasDependencies = false;
			object retval;
			string newID = "";
			string newDescription = "";
			
			if (MessageBox.Show("Are you sure you want to delete the selected item from the database. NOTE, this will completely delete the item and will affect all users.", "Confirm Delete",MessageBoxButtons.YesNo) == DialogResult.No)
				DeleteCancelled = true;
            
			if (!DeleteCancelled)
			{
				Processing p = new Processing();
				ArrayList UpdateSQLs = new ArrayList();
				try
				{
					retval = cMyDB.GetResult(lc.GetCheckUpdateDateSQL(Convert.ToString(SelectedValue),Text));

					if (retval == null || !retval.Equals(dt.Rows[SelectedIndex]["LastUpdated"]))
						throw new Exception("The list item you are trying to delete has been modified by another user and will not be deleted. Review the updated drop down list contents, if you still want to delete the entry you will be able to.");

					if (noOfDependents > 0)
					{
						p.progress.Minimum = 1;
						p.progress.Maximum = noOfDependents;
						p.Caption = "Checking the database for dependent values.";
						p.Show();
						p.Refresh();

						for (int i = p.progress.Minimum; i<(p.progress.Maximum + 1) ; i++)
						{
							p.progress.Value = i;
							if (!cMyDB.IsEmpty(lc.GetDependentCheckSQL(i - 1, Convert.ToString(SelectedValue), Text)))
							{
								if (!hasDependencies)
								{
									LookupReAssignForDeletion frmR = new LookupReAssignForDeletion();
									frmR.txtExisting.Text = this.Text;
									frmR.cboNewValue.LookupID = this.LookupID;
									frmR.cboNewValue.SelectedItem = null;
									//TODO: Add code to remove the item which will be deleted from the combo box
									while ((this.SelectedValue.Equals(frmR.cboNewValue.SelectedValue) || frmR.cboNewValue.SelectedIndex == -1) && !DeleteCancelled)
									{
										if (frmR.ShowDialog()==DialogResult.Cancel)
											DeleteCancelled = true;
										else
										{
											if (this.SelectedValue.Equals(frmR.cboNewValue.SelectedValue))
											{
												MessageBox.Show("The replacement you have selected is being deleted. Please select a different item.", "Invalid Selection");
												frmR.cboNewValue.SelectedIndex = -1;
											}
											else if (frmR.cboNewValue.SelectedIndex == -1)
												MessageBox.Show("You have not selected a replacement item. Either select a replacement or the deletion will be cancelled.", "Invalid Selection");
											else
											{
												hasDependencies = true;
												if (lc.HasIDField)
												{
													newID = Convert.ToString(frmR.cboNewValue.SelectedValue);
													newDescription = frmR.cboNewValue.Text;
												}
												else
												{
													newID = "";
													newDescription = frmR.cboNewValue.Text;
												}
											}
										}
									}
									frmR.Dispose();
								}
								UpdateSQLs.Add(lc.GetDependentUpdateSQL(i-1, Convert.ToString(SelectedValue), Text, newID, newDescription));
							}
						}
					}
					if (!DeleteCancelled)
					{
						p.Caption = "Updating Database";
						p.progress.Minimum = 1;
						p.progress.Maximum = 1 + UpdateSQLs.Count;
						p.progress.Value = 1;

						OleDbConnection cnn = new OleDbConnection(cMyDB.ConnectionString());
						OleDbTransaction tran;
						OleDbCommand cmd = new OleDbCommand();
						cnn.Open();
						tran = cnn.BeginTransaction();						

						try
						{
							cmd.Connection = cnn;
							cmd.Transaction = tran;

							if (hasDependencies && (UpdateSQLs.Count > 0))
							{
								foreach (object TempObj in UpdateSQLs)
								{
									cmd.CommandText = (string)TempObj;
									cmd.ExecuteNonQuery();
									p.progress.Value += 1;
								}
							}
							cmd.CommandText = lc.GetDeleteSQL(Convert.ToString(SelectedValue),Text);
							cmd.ExecuteNonQuery();
							
							tran.Commit();
							ItemDeleted = true;
						}
						catch (Exception dbe)
						{
							tran.Rollback();
							throw dbe;
						}
						finally
						{
							tran = null;
							cnn.Close();
							cnn.Dispose();
							cmd.Dispose();
						}
					}							
				}
				catch (Exception e)
				{
					MessageBox.Show("This item could not be deleted. " + e.Message, "Error");
				}
				finally
				{
					p.Close();
					UpdateSQLs = null;
				}
			}
			return ItemDeleted;
		}

		protected override void OnSelectionChangeCommitted(System.EventArgs e)
		{
			//implements custom behaviour if the special value "<new value>" is selected
			if (GetItemText(SelectedItem)== "<new value>")
			{
				if (lc.Updateable && AllowUpdates && (CSession.User.UserLevel >= lc.SecurityLevel))
				{
					if(!UpdateThenSelectNewItem(false))
						SelectedIndex = FindString(mCurrentSelection);  //select the original selection if there was one
				}
			}
			else
			{
				mCurrentIndex = this.SelectedIndex;
			}
			base.OnSelectionChangeCommitted(e);
		}

		private bool UpdateThenSelectNewItem(bool edit)
		{
			string newitem;
			
			if (edit)
				newitem = updateItem();
			else
				newitem = addNewItem();
			if (newitem!=null) //make sure something was added
			{
				RefreshDataSource();
				if (lc.Composite!="") //handle the special composite cases
				{
					string newcomp = (string)cMyDB.GetResult(lc.GetCompositeValueSQL(newitem));
					SelectedIndex = FindStringExact(newcomp);
				}
				else
				{
					SelectedIndex = FindStringExact(newitem);
				}
				return true;
			}
			else
			{
				if (ExtraRefreshes)
					RefreshDataSource();
				return false;
			}
		}

		protected override void OnDropDown(System.EventArgs e)
		{
			mCurrentIndex = this.SelectedIndex;
			base.OnDropDown(e);
		}

		private void mnu1_Popup_1(object sender, System.EventArgs e)
		{
			foreach (MenuItem m in mnu1.MenuItems)
				m.Enabled = true;
			mnuAdd.DefaultItem = true;
			if (!lc.Updateable || !AllowUpdates || (CSession.User.UserLevel < lc.SecurityLevel))
			{
				mnuAdd.Enabled = false;
				mnuEdit.Enabled = false;
				mnuDelete.Enabled = false;
				mnuAdd.DefaultItem = false;
				mnuNull.DefaultItem = true;
			}
			if (SelectedIndex == -1)
			{
				mnuDelete.Enabled = false;
				mnuEdit.Enabled = false;
				mnuNull.Enabled = false;
			}
			mnuNull.Visible = Nullable;
		}

		private void mnuAdd_Click(object sender, System.EventArgs e)
		{
			if (lc.Updateable && AllowUpdates)
			{
				if(!UpdateThenSelectNewItem(false))
					SelectedIndex = FindString(mCurrentSelection);
			}
		}

		private void mnuDelete_Click(object sender, System.EventArgs e)
		{
			if (lc.Updateable && AllowUpdates)
			{
				if(deleteItem())
				{
					SelectedIndex = -1;
					RefreshDataSource();
				}
			}
		}

		private void mnuEdit_Click(object sender, System.EventArgs e)
		{
			if (SelectedIndex == -1) 
				return;
			if (lc.Updateable && AllowUpdates)
			{
				if(!UpdateThenSelectNewItem(true))
					SelectedIndex = FindString(mCurrentSelection);
			}
		}

		private void mnuNull_Click(object sender, System.EventArgs e)
		{
			SelectedIndex = -1;
			if (ExtraRefreshes)
				RefreshDataSource();
		}

		public bool VerifySelectedValue()
		{
			if (SelectedValue != null && Text != null)
			{
				return !cMyDB.IsEmpty(lc.GetCheckUpdateDateSQL((string)SelectedValue, this.Text));
			}
			else
				return false;
		}
	}
}
