using System;

namespace DebtManagerTest
{
	/// <summary>
	/// Summary description for LookupConfig.
	/// </summary>
	public class LookupConfig
	{
		private string mID;
		private string mLookupTable;
		private string mAdditionalCheckField;
		private string mAdditionalCheckValue;
		private bool mAdditionalCheckIsString; //used by AdditionalCheckValue to add quotes
		private Int32 mLookupType;
		private string mExtraField;
		private bool mUpdateable;
		private Int32 mSecurityLevel;
		private string mComposite;
		private string mIDCaption;
		private string mDescriptionCaption;
		private string mAdditionalCaption;
		private string mAdditionalSpecial;
		private string mAdditionalSpecialParameter;
		private Int32 mMaxIDLength;
		private Int32 mMaxDescriptionLength;
		private Int32 mMaxAdditionalLength;
		private System.Data.DataTable mDependents;

		public void CleanUp()
		{
			mDependents.Dispose();
		}
		public LookupConfig()
		{
			
		}
		public LookupConfig(string xID, string xLookupTable,
			string xAdditionalCheckField,
			string xAdditionalCheckValue,
			bool xAdditionalCheckIsString,
			Int32 xLookupType,
			string xExtraField,
			bool xUpdateable,
			Int32 xSecurityLevel,
			string xComposite,
			string xIDCaption,
			string xDescriptionCaption,
			string xAdditionalCaption,
			string xAdditionalSpecial,
			string xAdditionalSpecialParameter,
			Int32 xMaxIDLength,
			Int32 xMaxDescriptionLength,
			Int32 xMaxAdditionalLength)
		{
			ID = xID;
			LookupTable = xLookupTable;
			AdditionalCheckField = xAdditionalCheckField;
			AdditionalCheckValue = xAdditionalCheckValue;
			AdditionalCheckIsString = xAdditionalCheckIsString;
			LookupType = xLookupType;
			ExtraField = xExtraField;
			Updateable = xUpdateable;
			SecurityLevel = xSecurityLevel;
			Composite = xComposite;
			IDCaption = xIDCaption;
			DescriptionCaption = xDescriptionCaption;
			AdditionalCaption = xAdditionalCaption;
			AdditionalSpecial = xAdditionalSpecial;
			AdditionalSpecialParameter = xAdditionalSpecialParameter;
			mMaxIDLength = xMaxIDLength;
			mMaxDescriptionLength = xMaxDescriptionLength;
			mMaxAdditionalLength = xMaxAdditionalLength;
		}
		public System.Data.DataTable Dependents
		{
			set
			{
				mDependents = value;
			}
		}
		public string ID 
		{
			get 
			{ 
				return mID; 
			}
			set 
			{ 
				mID = value;
			}
		}
		public string LookupTable
		{
			get 
			{ 
				return mLookupTable; 
			}
			set 
			{ 
				mLookupTable = value;
			}
		}
		public string AdditionalCheckField 
		{
			get 
			{ 
				return mAdditionalCheckField; 
			}
			set 
			{ 
				mAdditionalCheckField = value;
			}
		}
		public string AdditionalCheckValue 
		{
			get 
			{ 
				if (AdditionalCheckIsString)
					return "'" + mAdditionalCheckValue + "'"; 
				else
					return mAdditionalCheckValue; 
			}
			set 
			{ 
				mAdditionalCheckValue = value;
			}
		}
		public bool AdditionalCheckIsString 
		{
			get 
			{ 
				return mAdditionalCheckIsString;
			}
			set 
			{ 
				mAdditionalCheckIsString = value;
			}
		}
		public Int32 LookupType 
		{
			get 
			{ 
				return mLookupType; 
			}
			set 
			{ 
				mLookupType = value;
			}
		}
		public string ExtraField 
		{
			get 
			{ 
				return mExtraField; 
			}
			set 
			{ 
				mExtraField = value;
			}
		}
		public bool Updateable 
		{
			get 
			{ 
				return mUpdateable; 
			}
			set 
			{ 
				mUpdateable = value;
			}
		}
		public Int32 SecurityLevel
		{
			get 
			{ 
				return mSecurityLevel; 
			}
			set 
			{ 
				mSecurityLevel = value;
			}
		}
		public string Composite
		{
			get
			{
				return mComposite;
			}
			set 
			{
				mComposite = value;
			}
		}
		public string IDCaption
		{
			get
			{
				return mIDCaption;
			}
			set 
			{
				mIDCaption = value;
			}
		}
		public string DescriptionCaption
		{
			get
			{
				return mDescriptionCaption;
			}
			set 
			{
				mDescriptionCaption = value;
			}
		}
		public string AdditionalCaption
		{
			get
			{
				return mAdditionalCaption;
			}
			set 
			{
				mAdditionalCaption = value;
			}
		}
		public string AdditionalSpecial
		{
			get
			{
				return mAdditionalSpecial;
			}
			set 
			{
				mAdditionalSpecial = value;
			}
		}
		public string AdditionalSpecialParameter
		{
			get
			{
				return mAdditionalSpecialParameter;
			}
			set 
			{
				mAdditionalSpecialParameter = value;
			}
		}
		public bool HasIDField
		{
			get
			{
				if (LookupType > 1)
					return true;
				else
					return false;
			}
		}
		public string DuplicateKeyField
		{
			get
			{
				if (LookupType > 1 && LookupType <4)
					return "ID";
				else
					return "Description";
			}
		}
		public string SQL
		{
			get
			{
				string strWhere = "";
				string strExtraFrom = "";
				if (ExtraField!="")
				{
					strExtraFrom = ", " + ExtraField;
				}
				if (Composite!="")
				{
					strExtraFrom += ", " + Composite + " as CompositeValue";
				}
				if (AdditionalCheckField!="")
				{
					strWhere = " Where " + AdditionalCheckField + " = " + AdditionalCheckValue;
				}
				switch(LookupType)
				{
					case 1:
						return "SELECT Description " + strExtraFrom + ", LastUpdated, LastUser FROM " + LookupTable + strWhere;
					case 2:
					case 3:
					case 4:
						return "SELECT ID, Description " + strExtraFrom + ", LastUpdated, LastUser FROM " + LookupTable + strWhere;
					default:
						throw new System.ApplicationException("LookupType Incorrect");
				}
			}
		}
		public int ColumnCount
		{
			get
			{
				int i = 0;
				if (ExtraField!="")
					i++;
				if (Composite!="")
					i++;
				if (LookupType > 1)
					return 4 + i;
				else
					return 3 + i;
			}
		}
		public void FormatForm(LookupAddAndEditForm frm)
		{
			FormatForm(frm,false,null);
		}
		public void FormatForm(LookupAddAndEditForm frm, bool edit, System.Data.DataTable dt)
		{
			//formats the Add and Edit form
			if (!edit)
                frm.Text = "Enter New " + ID + " Details";
			else
				frm.Text = "Update " + ID + " Details";
			frm.lblDescription.Text = DescriptionCaption;
			if (mMaxIDLength > 0)
				frm.txtID.MaxLength = mMaxIDLength;
			if (mMaxDescriptionLength > 0)
				frm.txtDescription.MaxLength = mMaxDescriptionLength;
			if (mMaxAdditionalLength > 0 )
				frm.txtAdditional.MaxLength = mMaxAdditionalLength;
			switch(LookupType)
			{
				case 4:
					goto case 1;
				case 1:
					frm.txtID.Visible = false;
					frm.txtAdditional.Visible = false;
					
					if (edit)
						frm.txtDescription.DataBindings.Add("Text",dt,"Description");

					break;
				case 3:
					goto case 2;
				case 2:
					frm.lblID.Text = IDCaption;
					frm.lblDescription.Text = DescriptionCaption;
					
					if (edit)
					{
						frm.txtID.DataBindings.Add("Text",dt,"ID");
						frm.txtDescription.DataBindings.Add("Text",dt,"Description");
					}
					break;
			}
			if (AdditionalSpecial!="") //used to allow things like combo boxes for the additional value, or file selector dialogs
			{
				switch (AdditionalSpecial)
				{
					case "FILE":
						frm.cmdBrowse.Visible = true;
						frm.txtAdditional.Visible = true;
						if (edit)
							frm.txtAdditional.DataBindings.Add("Text",dt, ExtraField);
						break;
					case "COMBO":
						frm.cboAdditional.Location = frm.txtAdditional.Location;
						frm.cboAdditional.Size = frm.txtAdditional.Size;
						frm.txtAdditional.Visible = false;
						frm.cboAdditional.Visible = true;
						frm.cboAdditional.LookupID = AdditionalSpecialParameter;
						if (edit)
							frm.cboAdditional.DataBindings.Add("ComboValue",dt, ExtraField);
						break;
				}
			}
			else if (ExtraField!="")
			{
                frm.txtAdditional.Visible = true;
				if (edit)
					frm.txtAdditional.DataBindings.Add("Text",dt, ExtraField);                
			}
			frm.lblAdditional.Text = AdditionalCaption;
		}

		public string GetAdditionalValue(LookupAddAndEditForm frm)
		{
			if (ExtraField!="")
			{
				if (AdditionalSpecial != "COMBO")
				{
					return frm.txtAdditional.Text;
				}
				else
				{
					return (string)frm.cboAdditional.SelectedValue;
				}
			}
			else
				return frm.txtAdditional.Text;
		}

		public string GetDuplicateCheckSQL(string strID, string strDescription)
		{
			string strWhere;
			switch(LookupType)
			{
				case 1:
					strWhere = " WHERE Description = '" + strDescription + "'";
					return "SELECT Description FROM " + LookupTable + strWhere;
				case 2:
					strWhere = " WHERE ID = '" + strID + "'";
					return "SELECT ID FROM " + LookupTable + strWhere;
				case 3:
					strWhere = " WHERE ID = " + strID;
					return "SELECT ID FROM " + LookupTable + strWhere;
				case 4:
					if (AdditionalCheckField!="")
						strWhere = " WHERE Description = '" + strDescription + "' AND " + AdditionalCheckField + " = " + AdditionalCheckValue;
					else
                        strWhere = " WHERE Description = '" + strDescription + "'";
					return "SELECT Description FROM " + LookupTable + strWhere;
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
		}
		public string GetUpdateSQL(string curID, string newID, string curDescription, string newDescription, string strAdditional)
		{
			//TODO: Update when user code done
			string username = "test";
			string strWhere;
			string extra;
			if (ExtraField != "")
				extra = ", " + ExtraField + " = '" + strAdditional + "'";
			else
				extra = "";
			switch(LookupType)
			{
				case 1:
					strWhere = " WHERE Description = '" + curDescription + "'";
					return "UPDATE " + LookupTable + " SET Description = '" + 
						newDescription + "'" + extra + ",LastUpdated = Now,LastUser = '" + 
						username + "' " + strWhere;
				case 2:
					strWhere = " WHERE ID = '" + curID + "'";
					return "UPDATE " + LookupTable + " SET ID = '" + newID +
						"', Description = '" + newDescription + "'" + extra + 
						",LastUpdated = Now,LastUser = '" + 
						username + "' " + strWhere;
				case 3:
					strWhere = " WHERE ID = " + curID;
					return "UPDATE " + LookupTable + " SET ID = " + newID +
						", Description = '" + newDescription + "'" + extra + 
						",LastUpdated = Now,LastUser = '" + 
						username + "' " + strWhere;
				case 4:
					strWhere = " WHERE ID = " + curID ;
					return "UPDATE " + LookupTable + " SET Description = '" + 
						newDescription + "'" + extra + ",LastUpdated = Now,LastUser = '" + 
						username + "' " + strWhere;
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
		}
		public string GetCheckUpdateDateSQL(string strID, string strDescription)
		{
			string strWhere;
			switch(LookupType)
			{
				case 1:
					strWhere = " WHERE Description = '" + strDescription + "'";
					return "SELECT LastUpdated FROM " + LookupTable + strWhere;
				case 2:
					strWhere = " WHERE ID = '" + strID + "'";
					return "SELECT LastUpdated FROM " + LookupTable + strWhere;
				case 3:
					strWhere = " WHERE ID = " + strID;
					return "SELECT LastUpdated FROM " + LookupTable + strWhere;
				case 4:
					strWhere = " WHERE ID = " + strID;
					return "SELECT LastUpdated FROM " + LookupTable + strWhere;
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
		}
		public string GetCurrentValuesSQL(string strID, string strDescription)
		{
			string strWhere;

			if (AdditionalCheckField =="")
				strWhere = " WHERE ";
			else
				strWhere = " AND ";

			switch(LookupType)
			{
				case 1:
					strWhere += " Description = '" + strDescription + "'";
					break;
				case 2:
					strWhere += " ID = '" + strID + "'";
					break;
				case 3:
					strWhere += " ID = " + strID;
					break;
				case 4:
					strWhere += " ID = " + strID;
					break;
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
			return SQL + strWhere;
		}
		public string GetInsertSQL(string strID, string strDescription, string strAdditional)
		{
			//TODO: Update when user code done
			string username = "test";			
			string extra1;
			string extra2;
			if (ExtraField != "")
			{
				extra1 = ", " + ExtraField;
				extra2 = ", '" + strAdditional + "'"; 
			}
			else
			{
				extra1 = "";
				extra2 = "";
			}
			if (AdditionalCheckField!="")
			{
				extra1 += ", " + AdditionalCheckField;
				extra2 += ", " + AdditionalCheckValue;
			}
			switch(LookupType)
			{
				case 1:
					return "INSERT INTO " + LookupTable + 
						" (Description" + extra1 + ",LastUpdated,LastUser,Created,CreateUser) Values ('" +
						strDescription + "'" + extra2 + ",Now,'" + username + "',Now,'" + username + "')";
				case 2:
					return "INSERT INTO " + LookupTable + 
						" (ID,Description" + extra1 + ",LastUpdated,LastUser,Created,CreateUser) Values ('" +
						strID + "','" + strDescription + "'" + extra2 + ",Now,'" + username + "',Now,'" + username + "')";
				case 3:
					return "INSERT INTO " + LookupTable + 
						" (ID,Description" + extra1 + ",LastUpdated,LastUser,Created,CreateUser) Values (" + strID + 
						",'" +	strDescription + "'" + extra2 + ",Now,'" + username + "',Now,'" + username + "')";
				case 4:
					return "INSERT INTO " + LookupTable + 
						" (Description" + extra1 + ",LastUpdated,LastUser,Created,CreateUser) Values ('" + 
						strDescription + "'" + extra2 + ",Now,'" + username + "',Now,'" + username + "')";
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
		}
		public string GetDeleteSQL(string curID, string curDescription)
		{
			string strWhere;
			switch(LookupType)
			{
				case 1:
					strWhere = " WHERE Description = '" + curDescription + "'";
					return "DELETE FROM " + LookupTable + strWhere;
				case 2:
					strWhere = " WHERE ID = '" + curID + "'";
					return "DELETE FROM " + LookupTable + strWhere;
				case 3:
					strWhere = " WHERE ID = " + curID;
					return "DELETE FROM " + LookupTable + strWhere;
				case 4:
					strWhere = " WHERE ID = " + curID ;
					return "DELETE FROM " + LookupTable + strWhere;
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
		}

		public int DependentsCount
		{
			get
			{
				return mDependents.Rows.Count;
			}
		}
		public string GetDependentCheckSQL(int DependentNo, string strID, string strDescription)
		{
			string strWhere;
			switch(LookupType)
			{
				case 1:
					strWhere = "'" + strDescription + "'";
					break;
				case 2:
					strWhere = "'" + strID + "'";
					break;
				case 3:
					strWhere =  strID;
					break;
				case 4:
					strWhere = strID;
					break;
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
			return "SELECT " + mDependents.Rows[DependentNo]["FieldName"].ToString() +
				" FROM " + mDependents.Rows[DependentNo]["TableName"].ToString() + 
				" WHERE " + mDependents.Rows[DependentNo]["FieldName"].ToString() + " = " + strWhere;
		}
		public string GetDependentUpdateSQL(int DependentNo, string strID, string strDescription, string strNewID, string strNewDescription)
		{
			string strWhere;
			string strWhere2;
			switch(LookupType)
			{
				case 1:
					strWhere = "'" + strDescription + "'";
					strWhere2 = "'" + strNewDescription + "'";
					break;
				case 2:
					strWhere = "'" + strID + "'";
					strWhere2 = "'" + strNewID + "'";
					break;
				case 3:
					strWhere =  strID;
					strWhere2 =  strNewID;
					break;
				case 4:
					strWhere = strID;
					strWhere2 =  strNewID;
					break;
				default:
					throw new System.ApplicationException("LookupType Incorrect");
			}
			return "UPDATE " + mDependents.Rows[DependentNo]["TableName"].ToString() +
				" SET " + mDependents.Rows[DependentNo]["FieldName"].ToString() + " = " + strWhere2 +
				", LastUpdated = Now, LastUser = 'test'" + // TODO:Update when user code finished
				" WHERE " + mDependents.Rows[DependentNo]["FieldName"].ToString() + " = " + strWhere;
		}
		public string GetCompositeValueSQL(string strDescription)
		{
			if (Composite!="")
			{
				string strWhere;
				if (AdditionalCheckField!="")
				{
					strWhere = " AND " + AdditionalCheckField + " = " + AdditionalCheckValue;
				}
				else
				{
					strWhere = "";
				}
				return "Select " + Composite + " as CompositeValue From " + LookupTable + " Where Description = '" + strDescription + "' " + strWhere;
			}
			else
				throw new ApplicationException("Method called incorrectly. (GetCompositeValueSQL)");
		}
		public bool ValidateID(string strID)
		{
			if (strID!="")
			{
				if (LookupType == 3)
				{
					try
					{
						int intID = Convert.ToInt32(strID);
						return true;
					}
					catch 
					{
						return false;
					}
				}
				else
				{
					return true;
				}
			}
			else
			{
				if (LookupType > 1 && LookupType < 4)
					return false;
				else
					return true;
			}
		}
		public bool ValidateDescription(string strDescription)
		{
			if (strDescription!="")
				if (strDescription == "<new value>")
					return false;
				else
					return true;
			else
				return false;
		}
	}
}
