using System;
using System.Windows.Forms;

namespace DebtManagerTest
{
	/// <summary>
	/// Summary description for CurrencyBinding.
	/// </summary>
	public class ShortDateBinding : Binding
	{
		public ShortDateBinding(string PropertyName, Object DataSource , string DataMember)
			: base(PropertyName, DataSource, DataMember)
		{
		}
		
		protected override void OnFormat(ConvertEventArgs cevent)
		{
			/* This method is the Format event handler. Whenever the 
			   control displays a new value, the value is converted from 
			   its native Decimal type to a string. The ToString method 
			   then formats the value as a Currency, by using the 
			   formatting character "c". */

			// The application can only convert to string type. 
			if(cevent.DesiredType != typeof(string)) return;

			if(!cevent.Value.Equals(DBNull.Value))
				cevent.Value = ((DateTime) cevent.Value).ToShortDateString();
			base.OnFormat(cevent);
		}

		protected override void OnParse(ConvertEventArgs cevent)
		{	/* This method is the Parse event handler. The Parse event 
			   occurs whenever the displayed value changes. The static 
			   ToDecimal method of the Convert class converts the 
			   value back to its native Decimal type. */

			// Can only convert to decimal type.
			if(cevent.DesiredType != typeof(decimal)) return;

			if (cevent.Value.ToString()!="")
				cevent.Value = DateTime.Parse(cevent.Value.ToString());
			else
				cevent.Value = DBNull.Value;
			base.OnParse(cevent);
		}
	}
}
